'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');
var polyrhythm = require('polyrhythm');
var rxjs = require('rxjs');

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

/** Sets up a channel - a domain for events. It's not always necessary or
 * advantageous to use multiple channels - a single channel may suffice, and
 * different slices of it managed via namespacing.
 */

var ChannelContext = /*#__PURE__*/react.createContext(polyrhythm.channel);
/**
 * An alternative to the top-level static imports of useListener, trigger, etc.
 * that implicitly use the default channel. Uses a channel provided by a ChannelContext,
 * or falls back to the default channel.
 * @example: const { trigger, useListener } = useChannel();
 */

var useChannel = function useChannel() {
  var channel = react.useContext(ChannelContext) || polyrhythm.channel;
  return {
    channel: channel,
    trigger: function trigger(type, payload) {
      channel.trigger(type, payload);
    },
    query: function query(matcher) {
      return channel.query(matcher);
    },
    useListener: function useListener(eventSpec, handler, options, deps) {
      if (options === void 0) {
        options = {};
      }

      var _deps = deps || options.deps || [];

      react.useEffect(function () {
        var sub = channel.on(eventSpec, handler, options);
        return function () {
          return sub.unsubscribe();
        };
      }, _deps);
    },
    useFilter: function useFilter(eventSpec, filter, deps) {
      if (deps === void 0) {
        deps = [];
      }

      react.useEffect(function () {
        var sub = channel.filter(eventSpec, filter);
        return function () {
          return sub.unsubscribe();
        };
      }, deps);
    }
  };
};
/** Allows a component to attach (maybe async) consequences
 * to event patterns: strings, regexes, or function predicates.
 * Handlers return Observables for flexible scheduling/cancelation.
 * Cleans up when the component is unmounted.
 * @argument eventSpec - the criteria for the event to listen for
 * @argument handler - the function to be run upon the event, after all filters
 * @argument options - Specify the output events to be triggered in this object.
 * @returns Array of [trigger, unsub] - where trigger fires ANY event, and unsub cancels any inflight events (and is called automatcially on unmount).
 */

var useListener = function useListener(eventSpec, handler, options) {
  if (options === void 0) {
    options = {};
  }

  var _options = options,
      _options$deps = _options.deps,
      deps = _options$deps === void 0 ? [] : _options$deps,
      config = _objectWithoutPropertiesLoose(_options, ["deps"]);

  var _useChannel = useChannel(),
      channel = _useChannel.channel,
      trigger = _useChannel.trigger;

  var subscriptionRef = react.useRef(new rxjs.Subscription(function () {
    return null;
  }));
  var canceler;
  react.useEffect(function () {
    var subscription = channel.on(eventSpec, handler, config);
    subscriptionRef.current = subscription;

    canceler = function canceler() {
      subscription.unsubscribe();
      return subscription;
    };

    return canceler;
  }, deps); // @ts-ignore

  return [trigger, canceler];
};
/** Allows a component to intercept and run synchronous
   consequences, alter events, or throw errors to cancel the processing
   * @argument eventSpec - the criteria for the event to listen for
   * @argument filter - the function to be run synchronously upon the event before all listeners
   * @returns Array of [trigger, unsub] - where trigger fires the event, and unsub stops filtering, (and is called automatcially on unmount)
   */

var useFilter = function useFilter(eventSpec, filter) {
  var _useChannel2 = useChannel(),
      channel = _useChannel2.channel,
      trigger = _useChannel2.trigger;

  var canceler;
  react.useEffect(function () {
    var subscription = channel.filter(eventSpec, filter);

    canceler = function canceler() {
      subscription.unsubscribe();
      return subscription;
    };

    return function () {
      return subscription.unsubscribe();
    };
  }, []); // @ts-ignore

  return [trigger, canceler];
};

/** More readable version of useEffect(fn, []). Only on run at mount time. */

var useEffectAtMount = function useEffectAtMount(fn) {
  return react.useEffect(fn, []);
};
/** Run upon every render after the mount, subject to its deps */

var useEffectAfterMount = function useEffectAfterMount(func, deps) {
  if (deps === void 0) {
    deps = [];
  }

  var didMount = react.useRef(false);
  react.useEffect(function () {
    if (didMount.current !== false) {
      return func();
    } else {
      didMount.current = true;
    }
  }, deps);
};

/** Executes the Observable or Promise-returning listener in parallel (RxJS mergeMap) mode. */

function useASAPListener(matcher, listener, options) {
  if (options === void 0) {
    options = {};
  }

  var config = _extends({}, options, {
    mode: polyrhythm.ConcurrencyMode.parallel
  });

  return useListener(matcher, listener, config);
}
/** Executes the Observable or Promise-returning listener in a serial, queueing (RxJS concatMap) mode. */

function useQueuedListener(matcher, listener, options) {
  if (options === void 0) {
    options = {};
  }

  var config = _extends({}, options, {
    mode: polyrhythm.ConcurrencyMode.serial
  });

  return useListener(matcher, listener, config);
}
/** Executes the Observable or Promise-returning listener in a sliding, replacing (RxJS switchMap) mode. */

function useRestartingListener(matcher, listener, options) {
  if (options === void 0) {
    options = {};
  }

  var config = _extends({}, options, {
    mode: polyrhythm.ConcurrencyMode.replace
  });

  return useListener(matcher, listener, config);
}
/** Executes the Observable or Promise-returning listener in a singleton, ignoring (RxJS exhaustMap) mode. */

function useThrottledListener(matcher, listener, options) {
  if (options === void 0) {
    options = {};
  }

  var config = _extends({}, options, {
    mode: polyrhythm.ConcurrencyMode.ignore
  });

  return useListener(matcher, listener, config);
}
/** Executes the Observable or Promise-returning listener in a toggling mode. */

function useToggledListener(matcher, listener, options) {
  if (options === void 0) {
    options = {};
  }

  var config = _extends({}, options, {
    mode: polyrhythm.ConcurrencyMode.toggle
  });

  return useListener(matcher, listener, config);
}

var triggerAllMiddleware = function triggerAllMiddleware(channel) {
  if (channel === void 0) {
    channel = polyrhythm.channel;
  }

  return function () {
    return function (next) {
      return function (action) {
        // trigger first so channel spies and filters preceed everything
        channel.trigger(action);
        next(action);
      };
    };
  };
};

Object.defineProperty(exports, 'Channel', {
  enumerable: true,
  get: function () {
    return polyrhythm.Channel;
  }
});
Object.defineProperty(exports, 'after', {
  enumerable: true,
  get: function () {
    return polyrhythm.after;
  }
});
Object.defineProperty(exports, 'channel', {
  enumerable: true,
  get: function () {
    return polyrhythm.channel;
  }
});
Object.defineProperty(exports, 'concat', {
  enumerable: true,
  get: function () {
    return polyrhythm.concat;
  }
});
Object.defineProperty(exports, 'filter', {
  enumerable: true,
  get: function () {
    return polyrhythm.filter;
  }
});
Object.defineProperty(exports, 'listen', {
  enumerable: true,
  get: function () {
    return polyrhythm.listen;
  }
});
Object.defineProperty(exports, 'on', {
  enumerable: true,
  get: function () {
    return polyrhythm.on;
  }
});
Object.defineProperty(exports, 'query', {
  enumerable: true,
  get: function () {
    return polyrhythm.query;
  }
});
Object.defineProperty(exports, 'reset', {
  enumerable: true,
  get: function () {
    return polyrhythm.reset;
  }
});
Object.defineProperty(exports, 'spy', {
  enumerable: true,
  get: function () {
    return polyrhythm.spy;
  }
});
Object.defineProperty(exports, 'trigger', {
  enumerable: true,
  get: function () {
    return polyrhythm.trigger;
  }
});
exports.ChannelContext = ChannelContext;
exports.triggerAllMiddleware = triggerAllMiddleware;
exports.useASAPListener = useASAPListener;
exports.useChannel = useChannel;
exports.useEffectAfterMount = useEffectAfterMount;
exports.useEffectAtMount = useEffectAtMount;
exports.useFilter = useFilter;
exports.useListener = useListener;
exports.useQueuedListener = useQueuedListener;
exports.useRestartingListener = useRestartingListener;
exports.useThrottledListener = useThrottledListener;
exports.useToggledListener = useToggledListener;
//# sourceMappingURL=polyrhythm-react.cjs.development.js.map
