import { createContext, useContext, useEffect, useRef } from 'react';
import { channel, ConcurrencyMode } from 'polyrhythm';
export { Channel, after, channel, concat, filter, listen, on, query, reset, spy, trigger } from 'polyrhythm';
import { Subscription } from 'rxjs';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

/** Sets up a channel - a domain for events. It's not always necessary or
 * advantageous to use multiple channels - a single channel may suffice, and
 * different slices of it managed via namespacing.
 */

var ChannelContext = /*#__PURE__*/createContext(channel);
/**
 * An alternative to the top-level static imports of useListener, trigger, etc.
 * that implicitly use the default channel. Uses a channel provided by a ChannelContext,
 * or falls back to the default channel.
 * @example: const { trigger, useListener } = useChannel();
 */

var useChannel = function useChannel() {
  var channel$1 = useContext(ChannelContext) || channel;
  return {
    channel: channel$1,
    trigger: function trigger(type, payload) {
      channel$1.trigger(type, payload);
    },
    query: function query(matcher) {
      return channel$1.query(matcher);
    },
    useListener: function useListener(eventSpec, handler, options, deps) {
      if (options === void 0) {
        options = {};
      }

      var _deps = deps || options.deps || [];

      useEffect(function () {
        var sub = channel$1.on(eventSpec, handler, options);
        return function () {
          return sub.unsubscribe();
        };
      }, _deps);
    },
    useFilter: function useFilter(eventSpec, filter, deps) {
      if (deps === void 0) {
        deps = [];
      }

      useEffect(function () {
        var sub = channel$1.filter(eventSpec, filter);
        return function () {
          return sub.unsubscribe();
        };
      }, deps);
    }
  };
};
/** Allows a component to attach (maybe async) consequences
 * to event patterns: strings, regexes, or function predicates.
 * Handlers return Observables for flexible scheduling/cancelation.
 * Cleans up when the component is unmounted.
 * @argument eventSpec - the criteria for the event to listen for
 * @argument handler - the function to be run upon the event, after all filters
 * @argument options - Specify the output events to be triggered in this object.
 * @returns Array of [trigger, unsub] - where trigger fires ANY event, and unsub cancels any inflight events (and is called automatcially on unmount).
 */

var useListener = function useListener(eventSpec, handler, options) {
  if (options === void 0) {
    options = {};
  }

  var _options = options,
      _options$deps = _options.deps,
      deps = _options$deps === void 0 ? [] : _options$deps,
      config = _objectWithoutPropertiesLoose(_options, ["deps"]);

  var _useChannel = useChannel(),
      channel = _useChannel.channel,
      trigger = _useChannel.trigger;

  var subscriptionRef = useRef(new Subscription(function () {
    return null;
  }));
  var canceler;
  useEffect(function () {
    var subscription = channel.on(eventSpec, handler, config);
    subscriptionRef.current = subscription;

    canceler = function canceler() {
      subscription.unsubscribe();
      return subscription;
    };

    return canceler;
  }, deps); // @ts-ignore

  return [trigger, canceler];
};
/** Allows a component to intercept and run synchronous
   consequences, alter events, or throw errors to cancel the processing
   * @argument eventSpec - the criteria for the event to listen for
   * @argument filter - the function to be run synchronously upon the event before all listeners
   * @returns Array of [trigger, unsub] - where trigger fires the event, and unsub stops filtering, (and is called automatcially on unmount)
   */

var useFilter = function useFilter(eventSpec, filter) {
  var _useChannel2 = useChannel(),
      channel = _useChannel2.channel,
      trigger = _useChannel2.trigger;

  var canceler;
  useEffect(function () {
    var subscription = channel.filter(eventSpec, filter);

    canceler = function canceler() {
      subscription.unsubscribe();
      return subscription;
    };

    return function () {
      return subscription.unsubscribe();
    };
  }, []); // @ts-ignore

  return [trigger, canceler];
};

/** More readable version of useEffect(fn, []). Only on run at mount time. */

var useEffectAtMount = function useEffectAtMount(fn) {
  return useEffect(fn, []);
};
/** Run upon every render after the mount, subject to its deps */

var useEffectAfterMount = function useEffectAfterMount(func, deps) {
  if (deps === void 0) {
    deps = [];
  }

  var didMount = useRef(false);
  useEffect(function () {
    if (didMount.current !== false) {
      return func();
    } else {
      didMount.current = true;
    }
  }, deps);
};

/** Executes the Observable or Promise-returning listener in parallel (RxJS mergeMap) mode. */

function useASAPListener(matcher, listener, options) {
  if (options === void 0) {
    options = {};
  }

  var config = _extends({}, options, {
    mode: ConcurrencyMode.parallel
  });

  return useListener(matcher, listener, config);
}
/** Executes the Observable or Promise-returning listener in a serial, queueing (RxJS concatMap) mode. */

function useQueuedListener(matcher, listener, options) {
  if (options === void 0) {
    options = {};
  }

  var config = _extends({}, options, {
    mode: ConcurrencyMode.serial
  });

  return useListener(matcher, listener, config);
}
/** Executes the Observable or Promise-returning listener in a sliding, replacing (RxJS switchMap) mode. */

function useRestartingListener(matcher, listener, options) {
  if (options === void 0) {
    options = {};
  }

  var config = _extends({}, options, {
    mode: ConcurrencyMode.replace
  });

  return useListener(matcher, listener, config);
}
/** Executes the Observable or Promise-returning listener in a singleton, ignoring (RxJS exhaustMap) mode. */

function useThrottledListener(matcher, listener, options) {
  if (options === void 0) {
    options = {};
  }

  var config = _extends({}, options, {
    mode: ConcurrencyMode.ignore
  });

  return useListener(matcher, listener, config);
}
/** Executes the Observable or Promise-returning listener in a toggling mode. */

function useToggledListener(matcher, listener, options) {
  if (options === void 0) {
    options = {};
  }

  var config = _extends({}, options, {
    mode: ConcurrencyMode.toggle
  });

  return useListener(matcher, listener, config);
}

var triggerAllMiddleware = function triggerAllMiddleware(channel$1) {
  if (channel$1 === void 0) {
    channel$1 = channel;
  }

  return function () {
    return function (next) {
      return function (action) {
        // trigger first so channel spies and filters preceed everything
        channel$1.trigger(action);
        next(action);
      };
    };
  };
};

export { ChannelContext, triggerAllMiddleware, useASAPListener, useChannel, useEffectAfterMount, useEffectAtMount, useFilter, useListener, useQueuedListener, useRestartingListener, useThrottledListener, useToggledListener };
//# sourceMappingURL=polyrhythm-react.esm.js.map
